// bench-pow2.cpp
// solves the power-of-2 subproblem fast
// by Krasimir Georgiev
#include <iostream>
#include <cassert>

using namespace std;
typedef long long int bint;

int least_significant_0_idx(bint n) {
    int idx = 0;
    while ((n & 1) == 1) {
        n >>= 1;
        ++idx;
    }
    return idx;
}

int most_significant_1_idx(bint n) {
    int idx = 0;
    while (n != 0) {
        n >>= 1;
        ++idx;
    }
    return idx - 1;
}

bool is_pow_of_2(bint n) {
    return (n & (n - 1)) == 0;
}

bint man_idx(bint n, bint p) {
    return (n + p) >> (least_significant_0_idx(n + p) + 1);
}

bint man_pos(int k, bint i) {
    int shift = k - most_significant_1_idx(i);
    return (i << shift) + (bint(1) << (shift - 1)) - 1 - (bint(1) << k);
}

int main() {
    bint n, i, p;
    cin >> n >> i >> p;
    assert (is_pow_of_2(n + 1));

    int k = most_significant_1_idx(n) + 1;
    bint j = man_idx(n, p);
    bint q = 1 + man_pos(k, i);

    cout << q << ' ' << j << endl;
    return 0;
}
