/*
ID: espr1t
TASK: Bubbles
KEYWORDS: Medium, Ad-Hoc, NP
*/

#include <cstdio>
#include <cstring>
#include <cstdlib>
#include <ctime>
#include <vector>
#include <string>
#include <algorithm>

using namespace std;

const int MAX = 50;
const int MAX_DEPTH = 5;
const int MAX_SAVED = 15;

struct Cell {
    int row, col;
    Cell(int row_ = 0, int col_ = 0) : row(row_), col(col_) {}
};

struct State {
    Cell first;
    vector <string> board;
    int curScore, allScore;
    State(vector <string> board_ = vector<string>()) {
        board = board_;
        first = Cell(-1, -1);
        curScore = 0; allScore = 0;
    }
    bool operator < (const State& r) const {
        return allScore > r.allScore;
    }
    bool operator == (const State& r) const {
        return allScore == r.allScore && board == r.board;
    }
};

int numRows, numCols, numMoves, authorScore;
int vis[MAX][MAX];
int dir[4][2] = { {-1, 0}, {0, 1}, {1, 0}, {0, -1} };

inline int getScore(int num) {
    return num * num; // Change according to scoring algorithm
}

int flood(const vector <string>& board, int row, int col) {
    int ret = 1;
    vis[row][col] = 1;
    for (int i = 0; i < 4; i++) {
        int nrow = row + dir[i][0]; if (nrow < 0 || nrow >= numRows) continue;
        int ncol = col + dir[i][1]; if (ncol < 0 || ncol >= numCols) continue;
        if (board[row][col] == board[nrow][ncol] && !vis[nrow][ncol])
            ret += flood(board, nrow, ncol);
    }
    return ret;
}

int clickCell(vector <string>& board, Cell cell) {
    memset(vis, 0, sizeof(vis));
    int ret = getScore(flood(board, cell.row, cell.col));
    for (int col = 0; col < numCols; col++) {
        int nxt = 0;
        for (int row = 0; row < numRows; row++) {
            if (!vis[row][col])
                board[nxt++][col] = board[row][col];
        }
        while (nxt < numRows) board[nxt++][col] = '.';
    }
    return ret;
}

int greedy(vector <string> board, int rem) {
    vector <int> scores;
    memset(vis, 0, sizeof(vis));
    for (int row = 0; row < numRows; row++) {
        for (int col = 0; col < numCols; col++) {
            if (board[row][col] != '.' && !vis[row][col])
                scores.push_back(getScore(flood(board, row, col)));
        }
    }
    int ret = 0;
    sort(scores.rbegin(), scores.rend());
    for (int i = 0; i < min(rem, (int)scores.size()); i++)
        ret += scores[i];
    return ret;
}

vector <State> getStates(State state, int rem, int depth) {
    vector <Cell> click;
    memset(vis, 0, sizeof(vis));
    for (int row = 0; row < numRows; row++) {
        for (int col = 0; col < numCols; col++) {
            if (state.board[row][col] != '.' && !vis[row][col]) {
                flood(state.board, row, col);
                click.push_back(Cell(row, col));
            }
        }
    }
    vector <State> ret;
    for (int i = 0; i < (int)click.size(); i++) {
        State nstate = state;
        if (depth == 0) nstate.first = click[i];
        nstate.curScore += clickCell(nstate.board, click[i]);
        nstate.allScore = nstate.curScore;
        nstate.allScore += greedy(nstate.board, rem - depth - 1);
        ret.push_back(nstate);
    }
    sort(ret.begin(), ret.end());
    ret.resize(unique(ret.begin(), ret.end()) - ret.begin());
    ret.resize(min((int)ret.size(), MAX_SAVED));
    return ret;
}

vector <Cell> eval(vector <string> board) {
    vector <Cell> ans;
    for (int i = 0; i < numMoves; i++) {
//        fprintf(stderr, "    >> at move %d of %d...\n", i + 1, numMoves);
        vector <State> q1, q2;
        q1.push_back(State(board));
        int depth = min(numMoves - i, MAX_DEPTH);
        for (int c = 0; c < depth; c++) {
            for (int j = 0; j < (int)q1.size(); j++) {
                vector <State> states = getStates(q1[j], numMoves - i, c);
                q2.insert(q2.end(), states.begin(), states.end());
            }
            q1 = q2; q2.clear();
            sort(q1.begin(), q1.end());
            q1.resize(unique(q1.begin(), q1.end()) - q1.begin());
            q1.resize(min((int)q1.size(), MAX_SAVED));
        }
        clickCell(board, q1[0].first); ans.push_back(q1[0].first);
    }
    return ans;
}

int main(void) {
    FILE* in = fopen("Bubbles.in", "rt");
    FILE* out = fopen("Bubbles.out", "wt");
    
    int numTests;
    fscanf(in, "%d", &numTests);
    for (int test = 0; test < numTests; test++) {
        unsigned sTime = clock();
        
        vector <string> board;
        fscanf(in, "%d %d %d", &numRows, &numCols, &numMoves);
        for (int row = 0; row < numRows; row++) {
            char buff[MAX];
            fscanf(in, "%s", buff);
            board.push_back(buff);
        }
        reverse(board.begin(), board.end());
        fscanf(in, "%d", &authorScore);
    
        vector <Cell> answer = eval(board);
        for (int i = 0; i < (int)answer.size(); i++)
            fprintf(out, "%d %d\n", answer[i].row, answer[i].col);
    
        int score = 0;
        for (int i = 0; i < (int)answer.size(); i++) {
            if (answer[i].row < 0 || answer[i].col < 0)
                fprintf(stderr, "Uhm... negative coordinates!\n");
            if (board[answer[i].row][answer[i].col] == '.')
                fprintf(stderr, "Uuuuuhm... clicking on a dot?\n");
            score += clickCell(board, answer[i]);
        }
        fprintf(stderr, "  >> result: %d, exec time: %.3lf\n",
            score, (double)(clock() - sTime) / CLOCKS_PER_SEC);
    }
    return 0;
}

