/*
ID: espr1t
TASK: Bubbles
KEYWORDS: Medium, Ad-Hoc, NP
*/

#include <cstdio>
#include <cstring>
#include <cstdlib>
#include <ctime>
#include <vector>
#include <string>
#include <algorithm>

using namespace std;
FILE* in; FILE* out;

const int MAX = 50;
// These are dynamically adjusted.
int MAX_DEPTH = -1, MAX_SAVED = -1;
// Increasing this yields better results but linearly slower solution.
int MULTIPLIER = 50000;

struct Cell {
    int row, col;
    Cell(int row_ = 0, int col_ = 0) : row(row_), col(col_) {}
    bool operator < (const Cell& r) const {
        return row != r.row ? row < r.row : col < r.col;
    }
};

struct State {
    Cell first;
    vector <string> board;
    int curScore, allScore;
    State(vector <string> board_ = vector<string>()) {
        board = board_;
        first = Cell(-1, -1);
        curScore = 0; allScore = 0;
    }
    bool operator < (const State& r) const {
        return allScore > r.allScore;
    }
    bool operator == (const State& r) const {
        return allScore == r.allScore && board == r.board;
    }
};

int numMoves;
int numRows, numCols;
bool vis[MAX][MAX];
int dir[4][2] = { {-1, 0}, {0, 1}, {1, 0}, {0, -1} };

inline int getScore(int num) {
    return num * num; // Change according to scoring algorithm
}

int flood(const vector <string>& board, int row, int col) {
    int ret = 1;
    vis[row][col] = true;
    for (int i = 0; i < 4; i++) {
        int nrow = row + dir[i][0]; if (nrow < 0 || nrow >= numRows) continue;
        int ncol = col + dir[i][1]; if (ncol < 0 || ncol >= numCols) continue;
        if (board[row][col] == board[nrow][ncol] && !vis[nrow][ncol])
            ret += flood(board, nrow, ncol);
    }
    return ret;
}

int clickCell(vector <string>& board, Cell cell) {
    memset(vis, 0, sizeof(vis));
    int ret = getScore(flood(board, cell.row, cell.col));
    for (int col = 0; col < numCols; col++) {
        int nxt = 0;
        for (int row = 0; row < numRows; row++) {
            if (!vis[row][col])
                board[nxt++][col] = board[row][col];
        }
        while (nxt < numRows) board[nxt++][col] = '.';
    }
    return ret;
}

int greedy(vector <string> board, int rem) {
    vector <int> scores;
    memset(vis, 0, sizeof(vis));
    for (int row = 0; row < numRows; row++)
        for (int col = 0; col < numCols; col++)
            if (board[row][col] != '.' && !vis[row][col])
                scores.push_back(getScore(flood(board, row, col)));
    int ret = 0;
    sort(scores.rbegin(), scores.rend());
    for (int i = 0; i < min(rem, (int)scores.size()); i++)
        ret += scores[i];
    return ret;
}

vector <State> getStates(State state, int rem, int depth) {
    vector < pair <int, Cell> > click;
    memset(vis, 0, sizeof(vis));
    for (int row = 0; row < numRows; row++) {
        for (int col = 0; col < numCols; col++) {
            if (state.board[row][col] != '.' && !vis[row][col]) {
                int size = flood(state.board, row, col);
                click.push_back(make_pair(size, Cell(row, col)));
            }
        }
    }
    
    vector <State> ret;
    for (int i = 0; i < (int)click.size(); i++) {
        State nstate = state;
        if (depth == 0) nstate.first = click[i].second;
        nstate.curScore += clickCell(nstate.board, click[i].second);
        nstate.allScore = nstate.curScore;
        nstate.allScore += greedy(nstate.board, rem - depth - 1);
        ret.push_back(nstate);
    }
    sort(ret.begin(), ret.end());
    ret.resize(unique(ret.begin(), ret.end()) - ret.begin());
    ret.resize(min((int)ret.size(), MAX_SAVED));
    return ret;
}

vector <Cell> eval(vector <string> board) {
    MAX_DEPTH = 5;
    MAX_SAVED = MULTIPLIER / (numRows * numCols);

    vector <Cell> ans;
    for (int i = 0; i < numMoves; i++) {
        vector <State> q1, q2;
        q1.push_back(State(board));
        int depth = min(numMoves - i, MAX_DEPTH);
        for (int c = 0; c < depth; c++) {
            for (int j = 0; j < (int)q1.size(); j++) {
                vector <State> states = getStates(q1[j], numMoves - i, c);
                q2.insert(q2.end(), states.begin(), states.end());
            }
            sort(q2.begin(), q2.end());
            q2.resize(unique(q2.begin(), q2.end()) - q2.begin());
            q2.resize(min((int)q2.size(), MAX_SAVED));
            q1 = q2; q2.clear();
        }
        ans.push_back(q1[0].first);
        clickCell(board, ans.back());
    }
    return ans;
}

int main(void) {
    unsigned sTime = clock();
    
    in = stdin; out = stdout;
    //in = fopen("Bubbles.in", "rt"); out = fopen("Bubbles.out", "wt");

    vector <string> board;
    if (fscanf(in, "%d %d %d", &numRows, &numCols, &numMoves) != 3)
        fprintf(stderr, "Could not read 3 integers.\n");
    for (int row = 0; row < numRows; row++) {
        char buff[MAX];
        if (fscanf(in, "%s", buff) != 1)
            fprintf(stderr, "Could not read row %d\n", row);
        board.push_back(buff);
    }
    reverse(board.begin(), board.end());

    vector <Cell> answer = eval(board);
    for (int i = 0; i < (int)answer.size(); i++)
        fprintf(out, "%d %d\n", answer[i].row, answer[i].col);

    int score = 0;
    for (int i = 0; i < (int)answer.size(); i++) {
        if (answer[i].row < 0 || answer[i].col < 0)
            fprintf(stderr, "Uhm... negative coordinates!\n");
        if (board[answer[i].row][answer[i].col] == '.')
            fprintf(stderr, "Uuuuuhm... clicking on a dot?\n");
        score += clickCell(board, answer[i]);
    }
    fprintf(stderr, "  >> result: %d, exec time: %.3lf\n",
        score, (double)(clock() - sTime) / CLOCKS_PER_SEC);

    return 0;
}
