/*
ID: espr1t
TASK: Bubbles
KEYWORDS: Medium, Ad-Hoc, NP
*/

#include <cstdio>
#include <cstring>
#include <cstdlib>
#include <ctime>
#include <vector>
#include <string>
#include <algorithm>

using namespace std;
FILE* in; FILE* out;

const int MAX = 50;

struct Cell {
    int row, col;
    Cell(int row_ = 0, int col_ = 0) : row(row_), col(col_) {}
};

int numMoves;
int numRows, numCols;
int vis[MAX][MAX];
int dir[4][2] = { {-1, 0}, {0, 1}, {1, 0}, {0, -1} };

inline int getScore(int num) {
    return num * num; // Change according to scoring algorithm
}

int flood(const vector <string>& board, int row, int col, int color) {
    int ret = 1;
    vis[row][col] = color;
    for (int i = 0; i < 4; i++) {
        int nrow = row + dir[i][0]; if (nrow < 0 || nrow >= numRows) continue;
        int ncol = col + dir[i][1]; if (ncol < 0 || ncol >= numCols) continue;
        if (board[row][col] == board[nrow][ncol] && !vis[nrow][ncol])
            ret += flood(board, nrow, ncol, color);
    }
    return ret;
}

int clickCell(vector <string>& board, Cell cell) {
    memset(vis, 0, sizeof(vis));
    int ret = getScore(flood(board, cell.row, cell.col, 1));
    for (int col = 0; col < numCols; col++) {
        int nxt = numRows - 1;
        for (int row = numRows - 1; row >= 0; row--) {
            if (!vis[row][col])
                board[nxt--][col] = board[row][col];
        }
        while (nxt >= 0) board[nxt--][col] = '.';
    }
    return ret;
}

vector <Cell> eval(vector <string> board) {
    vector <Cell> ans;
    for (int i = 0; i < numMoves; i++) {
        Cell click(-1, -1); int win = 0;
        int nextCol = 1;
        memset(vis, 0, sizeof(vis));
        for (int row = 0; row < numRows; row++) {
            for (int col = 0; col < numCols; col++) {
                if (board[row][col] != '.' && !vis[row][col]) {
                    int score = getScore(flood(board, row, col, nextCol++));
                    if (win < score)
                        click = Cell(row, col), win = score;
                }
            }
        }
        if (win == 0) break;
        clickCell(board, click); ans.push_back(click);
    }
    return ans;
}

int main(void) {
    unsigned sTime = clock();

    in = stdin; out = stdout;
    //in = fopen("Bubbles.in", "rt"); out = fopen("Bubbles.out", "wt");

    vector <string> board;
    fscanf(in, "%d %d %d", &numRows, &numCols, &numMoves);
    for (int row = 0; row < numRows; row++) {
        char buff[MAX];
        fscanf(in, "%s", buff);
        board.push_back(buff);
    }
    reverse(board.begin(), board.end());

    vector <Cell> answer = eval(board);
    for (int i = 0; i < (int)answer.size(); i++)
        fprintf(out, "%d %d\n", answer[i].row, answer[i].col);

    int score = 0;
    for (int i = 0; i < (int)answer.size(); i++) {
        if (answer[i].row < 0 || answer[i].col < 0)
            fprintf(stderr, "Uhm... negative coordinates!\n");
        if (board[answer[i].row][answer[i].col] == '.')
            fprintf(stderr, "Uuuuuhm... clicking on a dot?\n");
        score += clickCell(board, answer[i]);
    }
    fprintf(stderr, "  >> result: %d, exec time: %.3lf\n",
        score, (double)(clock() - sTime) / CLOCKS_PER_SEC);
    return 0;
}

