/*
ID: espr1t
TASK: Bubbles
KEYWORDS: Checker
*/

#include <cstdio>
#include <cstring>
#include <cmath>

const int MAX = 64;
const int MIN_N = 1;
const int MAX_N = 50;
const int MIN_M = 1;
const int MAX_M = 50;
const int MIN_K = 1;
const int MAX_K = 50;
const int TEST_SCORE = 10;

FILE* out = stdout;
FILE* err = stderr;

char board[MAX][MAX];
int numRows, numCols, numMoves;
int authorScore, contestantScore;
int ans[MAX][2];

int click(int row, int col, char color) {
    int ret = 1;
    board[row][col] = ' ';

    int dir[4][2] = {{-1, 0}, {0, 1}, {1, 0}, {0, -1}};
    for (int i = 0; i < 4; i++) {
        int nrow = row + dir[i][0];
        if (nrow < 0 || nrow >= numRows) continue;
        int ncol = col + dir[i][1];
        if (ncol < 0 || ncol >= numCols) continue;
        if (board[nrow][ncol] == color)
            ret += click(nrow, ncol, color);
    }
    return ret;
}

void update() {
    for (int row = 0; row < numRows; row++) {
        for (int col = 0; col < numCols; col++) {
            if (board[row][col] != ' ') {
                int erow = row;
                while (erow > 0 && board[erow - 1][col] == ' ') {
                    board[erow - 1][col] = board[erow][col];
                    board[erow--][col] = ' ';
                }
            }
        }
    }
}

int evalSolution() {
    contestantScore = 0;
    for (int i = 0; i < numMoves; i++) {
        int row = ans[i][0], col = ans[i][1];
        if (board[row][col] == ' ')
            continue;
        int cells = click(row, col, board[row][col]);
        contestantScore += cells * cells;
        update();
    }
    return 0;
}


int getInput(char* fileName) {
    FILE* in = fopen(fileName, "rt");
    if (!in) {
        fprintf(out, "-1\n");
        fprintf(err, "Could not open input file!\n");
        return -1;
    }
    if (fscanf(in, "%d", &numRows) != 1) {
        fprintf(out, "-1\n");
        fprintf(err, "Could not read number of rows!\n");
        return -1;
    }
    if (fscanf(in, "%d", &numCols) != 1) {
        fprintf(out, "-1\n");
        fprintf(err, "Could not read number of columns!\n");
        return -1;
    }
    if (fscanf(in, "%d", &numMoves) != 1) {
        fprintf(out, "-1\n");
        fprintf(err, "Could not read number of rounds!\n");
        return -1;
    }
    for (int i = 0; i < numRows; i++) {
        if (fscanf(in, "%s", board[numRows - i - 1]) != 1) {
            fprintf(out, "-1\n");
            fprintf(err, "Could not read line %d of board!\n", i);
            return -1;
        }
    }
    if (fscanf(in, "%d", &authorScore) != 1) {
        fprintf(out, "-1\n");
        fprintf(err, "Could not read author's score!\n");
        return -1;
    }
    if (numRows < MIN_N || numRows > MAX_N) {
        fprintf(out, "-1\n");
        fprintf(err, "ERROR: Number of rows is invalid (%d)!\n", numRows);
        return -1;
    }
    if (numCols < MIN_M || numCols > MAX_M) {
        fprintf(out, "-1\n");
        fprintf(err, "ERROR: Number of columns is invalid (%d)!\n", numCols);
        return -1;
    }
    if (numMoves < MIN_K || numMoves > MAX_K) {
        fprintf(out, "-1\n");
        fprintf(err, "ERROR: Number of moves is invalid (%d)!\n", numMoves);
        return -1;
    }
    for (int row = 0; row < numRows; row++) {
        for (int col = 0; col < numCols; col++) {
            if (board[row][col] < 'A' || board[row][col] > 'Z') {
                fprintf(out, "-1\n");
                fprintf(err, "ERROR: Invalid character in board (%c)!\n",
                    board[row][col]);
                return -1;
            }
        }
    }
    return 0;
}

int getOutput(char* fileName) {
    FILE* in = fopen(fileName, "rt");
    if (!in) {
        fprintf(out, "0\n");
        fprintf(err, "Could not open output file!\n");
        return -1;
    }

    for (int i = 0; i < numMoves; i++) {
        if (fscanf(in, "%d %d", &ans[i][0], &ans[i][1]) != 2) {
            fprintf(out, "0\n");
            fprintf(err, "Could not read coordinates for %d-th move!\n", i);
            return -1;
        }
    }
    for (int i = 0; i < numMoves; i++) {
        if (ans[i][0] < 0 || ans[i][0] >= numRows) {
            fprintf(out, "0\n");
            fprintf(err, "Row must be in range [0, %d], but was %d!\n",
                numRows - 1, ans[i][0]);
            return -1;
        }
        if (ans[i][1] < 0 || ans[i][1] >= numCols) {
            fprintf(out, "0\n");
            fprintf(err, "Column must be in range [0, %d], but was %d!\n",
                numCols - 1, ans[i][1]);
            return -1;
        }
    }
    return 0;
}

int getScore(int a, int b) {
    if (b >= a) return TEST_SCORE;
    return round((double)b / (double)a * TEST_SCORE);
}

int main(int argc, char** argv) {
    if (argc != 3) {
        fprintf(stdout, "0\n");
        fprintf(stderr, "Usage: checker input_file output_file\n");
        return -1;
    }

    if (getInput(argv[1]) != 0)
        return -1;

    if (getOutput(argv[2]) != 0)
        return -1;

    if (evalSolution() != 0)
        return -1;

    fprintf(out, "%d\n", getScore(authorScore, contestantScore));
    fprintf(out, "OK (%d out of %d).\n", contestantScore, authorScore);
    
    return 0;
}
