/*
ID: espr1t
TASK: Bubbles
KEYWORDS: Testgen
*/

#include <cstdio>
#include <cstring>
#include <cstdlib>

const int MAX = 64;

int numRows, numCols, numMoves;
char board[MAX][MAX];

void sampleInput() {
    numRows = 5;
    numCols = 20;
    numMoves = 4;
    char a[64][64] = {
        "AAAEECCCCBBBAAEEPTTR",
        "AAAAECAACAABBAAAKAAA",
        "ACCCCAAESPRITBBBBAAA",
        "AZCCCAAFAAABBBASSSAA",
        "ACCCCAAAAAAAAAVAAPAA"
    };
    for (int i = 0; i < numRows; i++)
        for (int c = 0; c < numCols; c++)
            board[i][c] = a[i][c];
}

void clickOnSmallOnes() {
    numRows = 3;
    numCols = 20;
    numMoves = 21;
    char a[64][64] = {
        "AAAAAAAAAAAAAAAAAAAA",
        "BCBCBCBCBCBCBCBCBCBC",
        "AAAAAAAAAAAAAAAAAAAA"
    };
    for (int i = 0; i < numRows; i++)
        for (int c = 0; c < numCols; c++)
            board[i][c] = a[i][c];
}


void thinkAhead() {
    numRows = 5;
    numCols = 25;
    numMoves = 5;
    char a[64][64] = {
        "AAAAAAAAAAAAAAAAAAAAAAAAA",
        "BCBCBCBCBCAAAAACBCBCBCBCB",
        "BCBCBCBCBCBDBDBCBCBCBCBCB",
        "BCBCBCBCBCBCBCBCBCBCBCBCB",
        "AAAAAAAAAAAAAAAAAAAAAAAAA"
    };
    for (int i = 0; i < numRows; i++)
        for (int c = 0; c < numCols; c++)
            board[i][c] = a[i][c];
}

void horizontalLayers() {
    numRows = 29;
    numCols = 5;
    numMoves = 6;
    
    int row = 0;
    int height[] = {5, 4, 3, 2};
    for (int i = 0; i < 4; i++) {
        for (int c = 0; c < height[i]; c++) {
            for (int col = 0; col < numCols; col++)
                board[row][col] = i + 'A';
            row++;
        }
    }
    for (int col = 0; col < numCols; col++)
        board[row][col] = 'E';
    row++;
    for (int i = 3; i >= 0; i--) {
        for (int c = 0; c < height[i]; c++) {
            for (int col = 0; col < numCols; col++)
                board[row][col] = i + 'A';
            row++;
        }
    }
}

void handTest() {
    numRows = 50;
    numCols = 50;
    numMoves = 20;
    
    memset(board, 'A', sizeof(board));
    for (int row = 0; row < numRows; row++)
        board[row][numCols] = 0;

    for (int offRow = 0; offRow < 2; offRow++) {
        for (int offCol = 0; offCol < 2; offCol++) {
            int offR = offRow * 25;
            int offC = offCol * 25;
            char mainColor = offRow * 2 + offCol + 'A';
            for (int row = 0; row < 25; row++)
                for (int col = 0; col < 25; col++)
                    board[row + offR][col + offC] = mainColor;
            for (int row = 10; row < 15; row++)
                for (int col = 0; col < 25; col++)
                    board[row + offR][col + offC] = rand() % 25 + 'A';
        }
    }
}


void randomTest(int numRows_, int numCols_, int numMoves_, int numColors_) {
    numRows = numRows_;
    numCols = numCols_;
    numMoves = numMoves_;
    
    memset(board, 0, sizeof(board));
    for (int row = 0; row < numRows; row++) {
        for (int col = 0; col < numCols; col++) {
            board[row][col] = 'A' + rand() % numColors_;
        }
    }
    int blocks = numMoves / 4 + rand() % (numMoves / 4 + 1);
    for (int i = 0; i < blocks; i++) {
        int orow = rand() % numRows;
        int ocol = rand() % numCols;
        int power = rand() % 3 + 1;
        for (int row = 0; row < numRows; row++) {
            for (int col = 0; col < numCols; col++) {
                int dist = abs(row - orow) + abs(col - ocol);
                double threshold = (double)power / (double)(dist + 1);
                if ((double)rand() / (double)RAND_MAX < threshold)
                    board[row][col] = board[orow][ocol];
            }
        }
    }
}

void printTest(int testNum) {
    char fileName[32];
    sprintf(fileName, "Bubbles.%02d.in", testNum);
    FILE* out = fopen(fileName, "wt");
    fprintf(out, "%d %d %d\n", numRows, numCols, numMoves);
    for (int row = 0; row < numRows; row++) {
        for (int col = 0; col < numCols; col++)
            fprintf(out, "%c", board[row][col]);
        fprintf(out, "\n");
    }
    fclose(out);
}

int main(void) {
    srand(42);
    int numTests = 10;
    for (int test = 0; test <= numTests; test++) {
        if (test == 0) sampleInput();
        else if (test == 1) clickOnSmallOnes();
        else if (test == 2) thinkAhead();
        else if (test == 3) horizontalLayers();
        else if (test == 4) handTest();
        else if (test == 5) randomTest(20, 20, 5, 5);
        else if (test == 6) randomTest(30, 30, 50, 25);
        else if (test == 7) randomTest(40, 40, 20, 10);
        else if (test == 8) randomTest(50, 50, 25, 5);
        else if (test == 9) randomTest(50, 50, 50, 10);
        else if (test == 10) randomTest(50, 50, 50, 25);
        else fprintf(stderr, "Uhm..?\n");

        printTest(test);
    }
    return 0;
}
